import type { ArkErrors, arkKind } from "@ark/schema";
import type { inferred } from "@ark/util";
import type { GenericParser } from "../generic.ts";
import type { BoundModule, Module } from "../module.ts";
import { type Scope } from "../scope.ts";
import type { DeclarationParser, DefinitionParser, TypeParser } from "../type.ts";
import { arkBuiltins } from "./builtins.ts";
import { arkPrototypes } from "./constructors/constructors.ts";
import { number } from "./number/number.ts";
import { string } from "./string/string.ts";
import { arkTsGenerics, arkTsKeywords } from "./ts.ts";
export interface Ark extends Omit<Ark.keywords, keyof Ark.wrapped>, Ark.wrapped {
}
export declare namespace Ark {
    interface keywords extends arkTsKeywords.$, arkTsGenerics.$, arkPrototypes.keywords, arkBuiltins.$ {
    }
    interface wrapped extends arkPrototypes.wrapped {
        string: string.submodule;
        number: number.submodule;
    }
    interface typeAttachments extends arkTsKeywords.$ {
        Key: arkBuiltins.$["Key"];
        Record: arkTsGenerics.$["Record"];
    }
    interface boundTypeAttachments<$> extends Omit<BoundModule<typeAttachments, $>, arkKind> {
    }
}
export declare const ambient: Scope<Ark>;
export declare const ark: Module<Ark>;
export declare const type: TypeParser<{}>;
export declare namespace type {
    type cast<t> = {
        [inferred]?: t;
    };
    type errors = ArkErrors;
}
export declare const generic: GenericParser<{}>;
export declare const define: DefinitionParser<{}>;
export declare const declare: DeclarationParser<{}>;
