import { rootNode } from "@ark/schema";
import { registry } from "@ark/util";
import { submodule } from "../utils.js";
const value = rootNode(["string", registry.FileConstructor]);
const parsedFormDataValue = value.or(value.array());
const parsed = rootNode({
    meta: "an object representing parsed form data",
    domain: "object",
    index: {
        signature: "string",
        value: parsedFormDataValue
    }
});
export const arkFormData = submodule({
    $root: ["instanceof", FormData],
    value,
    parsed,
    parse: rootNode({
        in: FormData,
        morphs: (data) => {
            const result = {};
            for (const [k, v] of data) {
                if (k in result) {
                    const existing = result[k];
                    if (typeof existing === "string" ||
                        existing instanceof registry.FileConstructor)
                        result[k] = [existing, v];
                    else
                        existing.push(v);
                }
                else
                    result[k] = v;
            }
            return result;
        },
        declaredOut: parsed
    })
});
